@@ -1,11 +1,15 @@
-from functools import wraps
-from re import sub
+import os.path as op
 
-from flask import Blueprint, render_template, url_for, abort, g
+from functools import wraps
+
+from flask import Blueprint, current_app, render_template, abort, g, url_for
 from flask.ext.admin import babel
 from flask.ext.admin._compat import with_metaclass
 from flask.ext.admin import helpers as h
 
+# For compatibility reasons import MenuLink
+from flask.ext.admin.menu import MenuCategory, MenuView, MenuLink
+
 
 def expose(url='/', methods=('GET',)):
     """
@@ -47,6 +51,10 @@ def expose_plugview(url='/'):
 
 # Base views
 def _wrap_view(f):
+    # Avoid wrapping view method twice
+    if hasattr(f, '_wrapped'):
+        return f
+
     @wraps(f)
     def inner(self, *args, **kwargs):
         # Store current admin view
@@ -57,7 +65,9 @@ def _wrap_view(f):
         if abort is not None:
             return abort
 
-        return f(self, *args, **kwargs)
+        return self._run_view(f, *args, **kwargs)
+
+    inner._wrapped = True
 
     return inner
 
@@ -101,6 +111,7 @@ class BaseView(with_metaclass(AdminViewMeta, BaseViewClass)):
 
         Derive from this class to implement your administrative interface piece. For example::
 
+            from flask.ext.admin import BaseView, expose
             class MyView(BaseView):
                 @expose('/')
                 def index(self):
@@ -138,7 +149,8 @@ class BaseView(with_metaclass(AdminViewMeta, BaseViewClass)):
         return args
 
     def __init__(self, name=None, category=None, endpoint=None, url=None,
-                 static_folder=None, static_url_path=None):
+                 static_folder=None, static_url_path=None,
+                 menu_class_name=None, menu_icon_type=None, menu_icon_value=None):
         """
             Constructor.
 
@@ -158,9 +170,16 @@ class BaseView(with_metaclass(AdminViewMeta, BaseViewClass)):
                 and '/admin/' prefix won't be applied.
             :param static_url_path:
                 Static URL Path. If provided, this specifies the path to the static url directory.
-            :param debug:
-                Optional debug flag. If set to `True`, will rethrow exceptions in some cases, so Werkzeug
-                debugger can catch them.
+            :param menu_class_name:
+                Optional class name for the menu item.
+            :param menu_icon_type:
+                Optional icon. Possible icon types:
+
+                 - `flask.ext.admin.consts.ICON_TYPE_GLYPH` - Bootstrap glyph icon
+                 - `flask.ext.admin.consts.ICON_TYPE_IMAGE` - Image relative to Flask static directory
+                 - `flask.ext.admin.consts.ICON_TYPE_IMAGE_URL` - Image with full URL
+            :param menu_icon_value:
+                Icon glyph name or URL, depending on `menu_icon_type` setting
         """
         self.name = name
         self.category = category
@@ -168,6 +187,11 @@ class BaseView(with_metaclass(AdminViewMeta, BaseViewClass)):
         self.url = url
         self.static_folder = static_folder
         self.static_url_path = static_url_path
+        self.menu = None
+
+        self.menu_class_name = menu_class_name
+        self.menu_icon_type = menu_icon_type
+        self.menu_icon_value = menu_icon_value
 
         # Initialized from create_blueprint
         self.admin = None
@@ -211,13 +235,13 @@ class BaseView(with_metaclass(AdminViewMeta, BaseViewClass)):
 
         # If name is not povided, use capitalized endpoint name
         if self.name is None:
-            self.name = self._prettify_name(self.__class__.__name__)
+            self.name = self._prettify_class_name(self.__class__.__name__)
 
         # Create blueprint and register rules
         self.blueprint = Blueprint(self.endpoint, __name__,
                                    url_prefix=self.url,
                                    subdomain=self.admin.subdomain,
-                                   template_folder='templates',
+                                   template_folder=op.join('templates', self.admin.template_mode),
                                    static_folder=self.static_folder,
                                    static_url_path=self.static_url_path)
 
@@ -248,19 +272,25 @@ class BaseView(with_metaclass(AdminViewMeta, BaseViewClass)):
         kwargs['_ngettext'] = babel.ngettext
         kwargs['h'] = h
 
+        # Expose get_url helper
+        kwargs['get_url'] = self.get_url
+
+        # Expose config info
+        kwargs['config'] = current_app.config
+
         # Contribute extra arguments
         kwargs.update(self._template_args)
 
         return render_template(template, **kwargs)
 
-    def _prettify_name(self, name):
+    def _prettify_class_name(self, name):
         """
-            Prettify a class name by splitting the name on capitalized characters. So, 'MySuperClass' becomes 'My Super Class'
+            Split words in PascalCase string into separate words.
 
             :param name:
                 String to prettify
         """
-        return sub(r'(?<=.)([A-Z])', r' \1', name)
+        return h.prettify_class_name(name)
 
     def is_visible(self):
         """
@@ -288,8 +318,8 @@ class BaseView(with_metaclass(AdminViewMeta, BaseViewClass)):
         """
             This method will be executed before calling any view method.
 
-            By default, it will check if the admin class is accessible and if it is not it will
-            throw HTTP 404 error.
+            It will execute the ``inaccessible_callback`` if the view is not
+            accessible.
 
             :param name:
                 View function name
@@ -297,7 +327,43 @@ class BaseView(with_metaclass(AdminViewMeta, BaseViewClass)):
                 View function arguments
         """
         if not self.is_accessible():
-            return abort(403)
+            return self.inaccessible_callback(name, **kwargs)
+
+    def _run_view(self, fn, *args, **kwargs):
+        """
+            This method will run actual view function.
+
+            While it is similar to _handle_view, can be used to change
+            arguments that are passed to the view.
+
+            :param fn:
+                View function
+            :param kwargs:
+                Arguments
+        """
+        return fn(self, *args, **kwargs)
+
+    def inaccessible_callback(self, name, **kwargs):
+        """
+            Handle the response to inaccessible views.
+
+            By default, it throw HTTP 403 error. Override this method to
+            customize the behaviour.
+        """
+        return abort(403)
+
+    def get_url(self, endpoint, **kwargs):
+        """
+            Generate URL for the endpoint. If you want to customize URL generation
+            logic (persist some query string argument, for example), this is
+            right place to do it.
+
+            :param endpoint:
+                Flask endpoint name
+            :param kwargs:
+                Arguments for `url_for`
+        """
+        return url_for(endpoint, **kwargs)
 
     @property
     def _debug(self):
@@ -344,79 +410,6 @@ class AdminIndexView(BaseView):
         return self.render(self._template)
 
 
-class MenuItem(object):
-    """
-        Simple menu tree hierarchy.
-    """
-    def __init__(self, name, view=None):
-        self.name = name
-        self._view = view
-        self._children = []
-        self._children_urls = set()
-        self._cached_url = None
-
-        self.url = None
-        if view is not None:
-            self.url = view.url
-
-    def add_child(self, view):
-        self._children.append(view)
-        self._children_urls.add(view.url)
-
-    def get_url(self):
-        if self._view is None:
-            return None
-
-        if self._cached_url:
-            return self._cached_url
-
-        self._cached_url = url_for('%s.%s' % (self._view.endpoint, self._view._default_view))
-        return self._cached_url
-
-    def is_active(self, view):
-        if view == self._view:
-            return True
-
-        return view.url in self._children_urls
-
-    def is_visible(self):
-        if self._view is None:
-            return False
-
-        return self._view.is_visible()
-
-    def is_accessible(self):
-        if self._view is None:
-            return False
-
-        return self._view.is_accessible()
-
-    def is_category(self):
-        return self._view is None
-
-    def get_children(self):
-        return [c for c in self._children if c.is_accessible() and c.is_visible()]
-
-
-class MenuLink(object):
-    """
-        Additional menu links.
-    """
-    def __init__(self, name, url=None, endpoint=None):
-        self.name = name
-        self.url = url
-        self.endpoint = endpoint
-
-    def get_url(self):
-        return self.url or url_for(self.endpoint)
-
-    def is_visible(self):
-        return True
-
-    def is_accessible(self):
-        return True
-
-
 class Admin(object):
     """
         Collection of the admin views. Also manages menu structure.
@@ -427,7 +420,8 @@ class Admin(object):
                  translations_path=None,
                  endpoint=None,
                  static_url_path=None,
-                 base_template=None):
+                 base_template=None,
+                 template_mode=None):
         """
             Constructor.
 
@@ -452,6 +446,9 @@ class Admin(object):
                 all its views. Can be overridden in view configuration.
             :param base_template:
                 Override base HTML template for all static views. Defaults to `admin/base.html`.
+            :param template_mode:
+                Base template path. Defaults to `bootstrap2`. If you want to use
+                Bootstrap 3 integration, change it to `bootstrap3`.
         """
         self.app = app
 
@@ -472,6 +469,7 @@ class Admin(object):
         self.static_url_path = static_url_path
         self.subdomain = subdomain
         self.base_template = base_template or 'admin/base.html'
+        self.template_mode = template_mode or 'bootstrap2'
 
         # Add predefined index view
         self.add_view(self.index_view)
@@ -493,7 +491,8 @@ class Admin(object):
         # If app was provided in constructor, register view with Flask app
         if self.app is not None:
             self.app.register_blueprint(view.create_blueprint(self))
-            self._add_view_to_menu(view)
+
+        self._add_view_to_menu(view)
 
     def add_link(self, link):
         """
@@ -502,26 +501,36 @@ class Admin(object):
             :param link:
                 Link to add.
         """
-        self._menu_links.append(link)
+        if link.category:
+            self._add_menu_item(link, link.category)
+        else:
+            self._menu_links.append(link)
 
-    def _add_view_to_menu(self, view):
+    def _add_menu_item(self, menu_item, target_category):
         """
             Add a view to the menu tree
 
             :param view:
                 View to add
         """
-        if view.category:
-            category = self._menu_categories.get(view.category)
+        if target_category:
+            category = self._menu_categories.get(target_category)
 
             if category is None:
-                category = MenuItem(view.category)
-                self._menu_categories[view.category] = category
+                category = MenuCategory(target_category)
+                self._menu_categories[target_category] = category
+
                 self._menu.append(category)
 
-            category.add_child(MenuItem(view.name, view))
+            category.add_child(menu_item)
         else:
-            self._menu.append(MenuItem(view.name, view))
+            self._menu.append(menu_item)
+
+    def _add_view_to_menu(self, view):
+        self._add_menu_item(MenuView(view.name, view), view.category)
+
+    def get_category_menu_item(self, name):
+        return self._menu_categories.get(name)
 
     def init_app(self, app):
         """
@@ -537,7 +546,6 @@ class Admin(object):
         # Register views
         for view in self._views:
             app.register_blueprint(view.create_blueprint(self))
-            self._add_view_to_menu(view)
 
     def _init_extension(self):
         if not hasattr(self.app, 'extensions'):
