@@ -4,10 +4,11 @@ import platform
 import re
 import shutil
 
+from datetime import datetime
 from operator import itemgetter
 from werkzeug import secure_filename
 
-from flask import flash, url_for, redirect, abort, request, send_file
+from flask import flash, redirect, abort, request, send_file
 
 from wtforms import fields, validators
 
@@ -24,7 +25,7 @@ class NameForm(form.BaseForm):
 
         Validates if provided name is valid for *nix and Windows systems.
     """
-    name = fields.TextField()
+    name = fields.StringField()
 
     regexp = re.compile(r'^(?!^(PRN|AUX|CLOCK\$|NUL|CON|COM\d|LPT\d|\..*)(\..+)?$)[^\x00-\x1f\\?*:\";|/]+$')
 
@@ -302,14 +303,14 @@ class FileAdmin(BaseView, ActionsMixin):
                 Additional arguments
         """
         if not path:
-            return url_for(endpoint)
+            return self.get_url(endpoint)
         else:
             if self._on_windows:
                 path = path.replace('\\', '/')
 
             kwargs['path'] = path
 
-            return url_for(endpoint, **kwargs)
+            return self.get_url(endpoint, **kwargs)
 
     def _get_file_url(self, path):
         """
@@ -322,7 +323,8 @@ class FileAdmin(BaseView, ActionsMixin):
             route = '.edit'
         else:
             route = '.download'
-        return url_for(route, path=path)
+
+        return self.get_url(route, path=path)
 
     def _normalize_path(self, path):
         """
@@ -352,6 +354,8 @@ class FileAdmin(BaseView, ActionsMixin):
     def is_action_allowed(self, name):
         if name == 'delete' and not self.can_delete:
             return False
+        elif name == 'edit' and len(self.editable_extensions) == 0:
+            return False
 
         return True
 
@@ -458,7 +462,7 @@ class FileAdmin(BaseView, ActionsMixin):
             rel_path = op.join(path, f)
 
             if self.is_accessible_path(rel_path):
-                items.append((f, rel_path, op.isdir(fp), op.getsize(fp)))
+                items.append((f, rel_path, op.isdir(fp), op.getsize(fp), op.getmtime(fp)))
 
         # Sort by name
         items.sort(key=itemgetter(0))
@@ -466,6 +470,9 @@ class FileAdmin(BaseView, ActionsMixin):
         # Sort by type
         items.sort(key=itemgetter(2), reverse=True)
 
+        # Sort by modified date
+        items.sort(key=lambda values: (values[0], values[1], values[2], values[3], datetime.fromtimestamp(values[4])), reverse=True)
+
         # Generate breadcrumbs
         accumulator = []
         breadcrumbs = []
@@ -531,7 +538,7 @@ class FileAdmin(BaseView, ActionsMixin):
         # backward compatibility with base_url
         base_url = self.get_base_url()
         if base_url:
-            base_url = urljoin(url_for('.index'), base_url)
+            base_url = urljoin(self.get_url('.index'), base_url)
             return redirect(urljoin(base_url, path))
 
         return send_file(directory)
@@ -580,7 +587,7 @@ class FileAdmin(BaseView, ActionsMixin):
         path = request.form.get('path')
 
         if not path:
-            return redirect(url_for('.index'))
+            return redirect(self.get_url('.index'))
 
         # Get path and verify if it is valid
         base_path, full_path, path = self._normalize_path(path)
@@ -603,7 +610,7 @@ class FileAdmin(BaseView, ActionsMixin):
             try:
                 shutil.rmtree(full_path)
                 self.on_directory_delete(full_path, path)
-                flash(gettext('Directory "%s" was successfully deleted.' % path))
+                flash(gettext('Directory "%(path)s" was successfully deleted.', path=path))
             except Exception as ex:
                 flash(gettext('Failed to delete directory: %(error)s', error=ex), 'error')
         else:
@@ -624,7 +631,7 @@ class FileAdmin(BaseView, ActionsMixin):
         path = request.args.get('path')
 
         if not path:
-            return redirect(url_for('.index'))
+            return redirect(self.get_url('.index'))
 
         base_path, full_path, path = self._normalize_path(path)
 
@@ -669,13 +676,15 @@ class FileAdmin(BaseView, ActionsMixin):
         """
             Edit view method
         """
-        path = request.args.getlist('path')
         next_url = None
+
+        path = request.args.getlist('path')
         if not path:
-            return redirect(url_for('.index'))
+            return redirect(self.get_url('.index'))
 
         if len(path) > 1:
-            next_url = url_for('.edit', path=path[1:])
+            next_url = self.get_url('.edit', path=path[1:])
+
         path = path[0]
 
         base_path, full_path, path = self._normalize_path(path)
@@ -753,4 +762,4 @@ class FileAdmin(BaseView, ActionsMixin):
 
     @action('edit', lazy_gettext('Edit'))
     def action_edit(self, items):
-        return redirect(url_for('.edit', path=items))
+        return redirect(self.get_url('.edit', path=items))
