@@ -84,6 +84,22 @@ class ModelView(BaseModelView):
                 model_form_converter = MyModelConverter
     """
 
+    object_id_converter = ObjectId
+    """
+        Mongodb ``_id`` value conversion function. Default is `bson.ObjectId`.
+        Use this if you are using String, Binary and etc.
+
+        For example::
+
+            class MyModelView(BaseModelView):
+                object_id_converter = int
+
+        or::
+
+            class MyModelView(BaseModelView):
+                object_id_converter = str
+    """
+
     filter_converter = FilterConverter()
     """
         Field to filter converter.
@@ -183,7 +199,8 @@ class ModelView(BaseModelView):
     """
 
     def __init__(self, model, name=None,
-                 category=None, endpoint=None, url=None):
+                 category=None, endpoint=None, url=None, static_folder=None,
+                 menu_class_name=None, menu_icon_type=None, menu_icon_value=None):
         """
             Constructor
 
@@ -197,10 +214,24 @@ class ModelView(BaseModelView):
                 Endpoint
             :param url:
                 Custom URL
+            :param menu_class_name:
+                Optional class name for the menu item.
+            :param menu_icon_type:
+                Optional icon. Possible icon types:
+
+                 - `flask.ext.admin.consts.ICON_TYPE_GLYPH` - Bootstrap glyph icon
+                 - `flask.ext.admin.consts.ICON_TYPE_IMAGE` - Image relative to Flask static directory
+                 - `flask.ext.admin.consts.ICON_TYPE_IMAGE_URL` - Image with full URL
+
+            :param menu_icon_value:
+                Icon glyph name or URL, depending on `menu_icon_type` setting
         """
         self._search_fields = []
 
-        super(ModelView, self).__init__(model, name, category, endpoint, url)
+        super(ModelView, self).__init__(model, name, category, endpoint, url, static_folder,
+                                        menu_class_name=menu_class_name,
+                                        menu_icon_type=menu_icon_type,
+                                        menu_icon_value=menu_icon_value)
 
         self._primary_key = self.scaffold_pk()
 
@@ -400,9 +431,9 @@ class ModelView(BaseModelView):
 
         # Filters
         if self._filters:
-            for flt, value in filters:
+            for flt, flt_name, value in filters:
                 f = self._filters[flt]
-                query = f.apply(query, value)
+                query = f.apply(query, f.clean(value))
 
         # Search
         if self._search_supported and search:
@@ -476,13 +507,12 @@ class ModelView(BaseModelView):
             self._on_model_change(form, model, True)
             model.save()
         except Exception as ex:
-            if self._debug:
-                raise
+            if not self.handle_view_exception(ex):
+                flash(gettext('Failed to create record. %(error)s',
+                              error=format_error(ex)),
+                      'error')
+                log.exception('Failed to create record.')
 
-            flash(gettext('Failed to create model. %(error)s',
-                          error=format_error(ex)),
-                  'error')
-            log.exception('Failed to create model')
             return False
         else:
             self.after_model_change(form, model, True)
@@ -503,13 +533,12 @@ class ModelView(BaseModelView):
             self._on_model_change(form, model, False)
             model.save()
         except Exception as ex:
-            if self._debug:
-                raise
+            if not self.handle_view_exception(ex):
+                flash(gettext('Failed to update record. %(error)s',
+                              error=format_error(ex)),
+                      'error')
+                log.exception('Failed to update record.')
 
-            flash(gettext('Failed to update model. %(error)s',
-                          error=format_error(ex)),
-                  'error')
-            log.exception('Failed to update model')
             return False
         else:
             self.after_model_change(form, model, False)
@@ -528,13 +557,12 @@ class ModelView(BaseModelView):
             model.delete()
             return True
         except Exception as ex:
-            if self._debug:
-                raise
+            if not self.handle_view_exception(ex):
+                flash(gettext('Failed to delete record. %(error)s',
+                              error=format_error(ex)),
+                      'error')
+                log.exception('Failed to delete record.')
 
-            flash(gettext('Failed to delete model. %(error)s',
-                          error=format_error(ex)),
-                  'error')
-            log.exception('Failed to delete model')
             return False
 
     # FileField access API
@@ -549,7 +577,7 @@ class ModelView(BaseModelView):
 
         fs = gridfs.GridFS(get_db(db), coll)
 
-        data = fs.get(ObjectId(pk))
+        data = fs.get(self.object_id_converter(pk))
         if not data:
             abort(404)
 
@@ -569,22 +597,20 @@ class ModelView(BaseModelView):
 
     @action('delete',
             lazy_gettext('Delete'),
-            lazy_gettext('Are you sure you want to delete selected models?'))
+            lazy_gettext('Are you sure you want to delete selected records?'))
     def action_delete(self, ids):
         try:
             count = 0
 
-            all_ids = [ObjectId(pk) for pk in ids]
+            all_ids = [self.object_id_converter(pk) for pk in ids]
             for obj in self.get_query().in_bulk(all_ids).values():
                 count += self.delete_model(obj)
 
-            flash(ngettext('Model was successfully deleted.',
-                           '%(count)s models were successfully deleted.',
+            flash(ngettext('Record was successfully deleted.',
+                           '%(count)s records were successfully deleted.',
                            count,
                            count=count))
         except Exception as ex:
-            if self._debug:
-                raise
-
-            flash(gettext('Failed to delete models. %(error)s', error=str(ex)),
-                  'error')
+            if not self.handle_view_exception(ex):
+                flash(gettext('Failed to delete records. %(error)s', error=str(ex)),
+                      'error')
