@@ -1,9 +1,9 @@
 import warnings
 import re
 
-from flask import request, url_for, redirect, flash, abort, json, Response
-
+from flask import request, redirect, flash, abort, json, Response
 from jinja2 import contextfunction
+from wtforms.validators import ValidationError
 
 from flask.ext.admin.babel import gettext
 
@@ -14,7 +14,7 @@ from flask.ext.admin.actions import ActionsMixin
 from flask.ext.admin.helpers import get_form_data, validate_form_on_submit, get_redirect_target
 from flask.ext.admin.tools import rec_getattr
 from flask.ext.admin._backwards import ObsoleteAttr
-from flask.ext.admin._compat import iteritems, OrderedDict
+from flask.ext.admin._compat import iteritems, OrderedDict, as_unicode
 from .helpers import prettify_name, get_mdict_item_or_list
 from .ajax import AjaxModelLoader
 
@@ -24,6 +24,38 @@ filter_char_re = re.compile('[^a-z0-9 ]')
 filter_compact_re = re.compile(' +')
 
 
+class ViewArgs(object):
+    """
+        List view arguments.
+    """
+    def __init__(self, page=None, sort=None, sort_desc=None, search=None, filters=None, extra_args=None):
+        self.page = page
+        self.sort = sort
+        self.sort_desc = bool(sort_desc)
+        self.search = search
+        self.filters = filters
+
+        if not self.search:
+            self.search = None
+
+        self.extra_args = extra_args or dict()
+
+    def clone(self, **kwargs):
+        if self.filters:
+            flt = list(self.filters)
+        else:
+            flt = None
+
+        kwargs.setdefault('page', self.page)
+        kwargs.setdefault('sort', self.sort)
+        kwargs.setdefault('sort_desc', self.sort_desc)
+        kwargs.setdefault('search', self.search)
+        kwargs.setdefault('filters', flt)
+        kwargs.setdefault('extra_args', dict(self.extra_args))
+
+        return ViewArgs(**kwargs)
+
+
 class BaseModelView(BaseView, ActionsMixin):
     """
         Base model view.
@@ -282,7 +314,7 @@ class BaseModelView(BaseView, ActionsMixin):
         For example::
 
             class MyForm(Form):
-                name = TextField('Name')
+                name = StringField('Name')
 
             class MyModelView(BaseModelView):
                 form = MyForm
@@ -312,6 +344,7 @@ class BaseModelView(BaseView, ActionsMixin):
 
         Example::
 
+            from wtforms.validators import required
             class MyModelView(BaseModelView):
                 form_args = dict(
                     name=dict(label='First Name', validators=[required()])
@@ -365,6 +398,17 @@ class BaseModelView(BaseView, ActionsMixin):
                         'style': 'color: black'
                     }
                 }
+
+        Changing the format of a DateTimeField will require changes to both form_widget_args and form_args.
+
+        Example::
+
+            form_args = dict(
+                start=dict(format='%Y-%m-%d %I:%M %p') # changes how the input is parsed by strptime (12 hour time)
+            )
+            form_widget_args = dict(
+                start={'data-date-format': u'yyyy-mm-dd HH:ii P', 'data-show-meridian': 'True'} # changes how the DateTimeField displays the time
+            )
     """
 
     form_extra_fields = None
@@ -475,7 +519,8 @@ class BaseModelView(BaseView, ActionsMixin):
     """
 
     def __init__(self, model,
-                 name=None, category=None, endpoint=None, url=None):
+                 name=None, category=None, endpoint=None, url=None, static_folder=None,
+                 menu_class_name=None, menu_icon_type=None, menu_icon_value=None):
         """
             Constructor.
 
@@ -491,20 +536,30 @@ class BaseModelView(BaseView, ActionsMixin):
                 'userview'
             :param url:
                 Base URL. If not provided, will use endpoint as a URL.
-            :param debug:
-                Enable debugging mode. Won't catch exceptions on model
-                save failures.
+            :param menu_class_name:
+                Optional class name for the menu item.
+            :param menu_icon_type:
+                Optional icon. Possible icon types:
+
+                 - `flask.ext.admin.consts.ICON_TYPE_GLYPH` - Bootstrap glyph icon
+                 - `flask.ext.admin.consts.ICON_TYPE_IMAGE` - Image relative to Flask static directory
+                 - `flask.ext.admin.consts.ICON_TYPE_IMAGE_URL` - Image with full URL
+            :param menu_icon_value:
+                Icon glyph name or URL, depending on `menu_icon_type` setting
         """
 
         # If name not provided, it is model name
         if name is None:
             name = '%s' % self._prettify_class_name(model.__name__)
 
-        # If endpoint not provided, it is model name + 'view'
+        # If endpoint not provided, it is model name
         if endpoint is None:
-            endpoint = ('%sview' % model.__name__).lower()
+            endpoint = model.__name__.lower()
 
-        super(BaseModelView, self).__init__(name, category, endpoint, url)
+        super(BaseModelView, self).__init__(name, category, endpoint, url, static_folder,
+                                            menu_class_name=menu_class_name,
+                                            menu_icon_type=menu_icon_type,
+                                            menu_icon_value=menu_icon_value)
 
         self.model = model
 
@@ -515,18 +570,7 @@ class BaseModelView(BaseView, ActionsMixin):
         self._refresh_cache()
 
     # Caching
-    def _refresh_cache(self):
-        """
-            Refresh various cached variables.
-        """
-        # List view
-        self._list_columns = self.get_list_columns()
-        self._sortable_columns = self.get_sortable_columns()
-
-        # Labels
-        if self.column_labels is None:
-            self.column_labels = {}
-
+    def _refresh_forms_cache(self):
         # Forms
         self._form_ajax_refs = self._process_ajax_references()
 
@@ -536,29 +580,9 @@ class BaseModelView(BaseView, ActionsMixin):
         self._create_form_class = self.get_create_form()
         self._edit_form_class = self.get_edit_form()
 
-        # Search
-        self._search_supported = self.init_search()
-
-        # Choices
-        if self.column_choices:
-            self._column_choices_map = dict([
-                (column, dict(choices))
-                for column, choices in self.column_choices.items()
-            ])
-        else:
-            self.column_choices = self._column_choices_map = dict()
-
-        # Filters
+    def _refresh_filters_cache(self):
         self._filters = self.get_filters()
 
-        # Type formatters
-        if self.column_type_formatters is None:
-            self.column_type_formatters = dict(typefmt.BASE_FORMATTERS)
-
-        if self.column_descriptions is None:
-            self.column_descriptions = dict()
-
-        # Group filters by field name
         if self._filters:
             self._filter_groups = OrderedDict()
             self._filter_args = {}
@@ -570,7 +594,7 @@ class BaseModelView(BaseView, ActionsMixin):
                 self._filter_groups[flt.name].append({
                     'index': i,
                     'arg': self.get_filter_arg(i, flt),
-                    'operation': flt.operation(),
+                    'operation': as_unicode(flt.operation()),
                     'options': flt.get_options(self) or None,
                     'type': flt.data_type
                 })
@@ -580,7 +604,7 @@ class BaseModelView(BaseView, ActionsMixin):
             self._filter_groups = None
             self._filter_args = None
 
-        # Form rendering rules
+    def _refresh_form_rules_cache(self):
         if self.form_create_rules:
             self._form_create_rules = rules.RuleSet(self, self.form_create_rules)
         else:
@@ -600,12 +624,52 @@ class BaseModelView(BaseView, ActionsMixin):
             if not self._form_edit_rules:
                 self._form_edit_rules = form_rules
 
+    def _refresh_cache(self):
+        """
+            Refresh various cached variables.
+        """
+        # List view
+        self._list_columns = self.get_list_columns()
+        self._sortable_columns = self.get_sortable_columns()
+
+        # Labels
+        if self.column_labels is None:
+            self.column_labels = {}
+
+        # Forms
+        self._refresh_forms_cache()
+
+        # Search
+        self._search_supported = self.init_search()
+
+        # Choices
+        if self.column_choices:
+            self._column_choices_map = dict([
+                (column, dict(choices))
+                for column, choices in self.column_choices.items()
+            ])
+        else:
+            self.column_choices = self._column_choices_map = dict()
+
+        # Type formatters
+        if self.column_type_formatters is None:
+            self.column_type_formatters = dict(typefmt.BASE_FORMATTERS)
+
+        if self.column_descriptions is None:
+            self.column_descriptions = dict()
+
+        # Filters
+        self._refresh_filters_cache()
+
+        # Form rendering rules
+        self._refresh_form_rules_cache()
+
     # Primary key
     def get_pk_value(self, model):
         """
             Return PK value from a model object.
         """
-        raise NotImplemented()
+        raise NotImplementedError()
 
     # List view
     def scaffold_list_columns(self):
@@ -618,7 +682,7 @@ class BaseModelView(BaseView, ActionsMixin):
 
                 ['name', 'first_name', 'last_name']
         """
-        raise NotImplemented('Please implement scaffold_list_columns method')
+        raise NotImplementedError('Please implement scaffold_list_columns method')
 
     def get_column_name(self, field):
         """
@@ -657,7 +721,7 @@ class BaseModelView(BaseView, ActionsMixin):
             Expected return format is a dictionary, where keys are field names and
             values are property names.
         """
-        raise NotImplemented('Please implement scaffold_sortable_columns method')
+        raise NotImplementedError('Please implement scaffold_sortable_columns method')
 
     def get_sortable_columns(self):
         """
@@ -709,6 +773,15 @@ class BaseModelView(BaseView, ActionsMixin):
         """
         return isinstance(filter, filters.BaseFilter)
 
+    def handle_filter(self, filter):
+        """
+            Postprocess (add joins, etc) for a filter.
+
+            :param filter:
+                Filter object to postprocess
+        """
+        return filter
+
     def get_filters(self):
         """
             Return a list of filter objects.
@@ -721,7 +794,7 @@ class BaseModelView(BaseView, ActionsMixin):
 
             for n in self.column_filters:
                 if self.is_valid_filter(n):
-                    collection.append(n)
+                    collection.append(self.handle_filter(n))
                 else:
                     flt = self.scaffold_filters(n)
                     if flt:
@@ -745,7 +818,7 @@ class BaseModelView(BaseView, ActionsMixin):
                 Filter instance
         """
         if self.named_filter_urls:
-            name = ('%s %s' % (flt.name, flt.operation())).lower()
+            name = ('%s %s' % (flt.name, as_unicode(flt.operation()))).lower()
             name = filter_char_re.sub('', name)
             name = filter_compact_re.sub('_', name)
             return name
@@ -758,7 +831,7 @@ class BaseModelView(BaseView, ActionsMixin):
             Create `form.BaseForm` inherited class from the model. Must be
             implemented in the child class.
         """
-        raise NotImplemented('Please implement scaffold_form method')
+        raise NotImplementedError('Please implement scaffold_form method')
 
     def get_form(self):
         """
@@ -806,6 +879,15 @@ class BaseModelView(BaseView, ActionsMixin):
         """
         return self._edit_form_class(get_form_data(), obj=obj)
 
+    def validate_form(self, form):
+        """
+            Validate the form on submit.
+
+            :param form:
+                Form to validate
+        """
+        return validate_form_on_submit(form)
+
     # Helpers
     def is_sortable(self, name):
         """
@@ -856,7 +938,7 @@ class BaseModelView(BaseView, ActionsMixin):
                 List of filter tuples. First value in a tuple is a search
                 index, second value is a search value.
         """
-        raise NotImplemented('Please implement get_list method')
+        raise NotImplementedError('Please implement get_list method')
 
     def get_one(self, id):
         """
@@ -867,7 +949,18 @@ class BaseModelView(BaseView, ActionsMixin):
             :param id:
                 Model id
         """
-        raise NotImplemented('Please implement get_one method')
+        raise NotImplementedError('Please implement get_one method')
+
+    # Exception handler
+    def handle_view_exception(self, exc):
+        if isinstance(exc, ValidationError):
+            flash(as_unicode(exc))
+            return True
+
+        if self._debug:
+            raise
+
+        return False
 
     # Model event handlers
     def on_model_change(self, form, model, is_created):
@@ -930,6 +1023,29 @@ class BaseModelView(BaseView, ActionsMixin):
         """
         pass
 
+    def on_form_prefill (self, form, id):
+        """
+            Perform additional actions to pre-fill the edit form.
+
+            Called from edit_view, if the current action is rendering
+            the form rather than receiving client side input, after
+            default pre-filling has been performed.
+
+            By default does nothing.
+
+            You only need to override this if you have added custom
+            fields that depend on the database contents in a way that
+            Flask-admin can't figure out by itself. Fields that were
+            added by name of a normal column or relationship should
+            work out of the box.
+
+            :param form:
+                Form instance
+            :param id:
+                id of the object that is going to be edited
+        """
+        pass
+
     def create_model(self, form):
         """
             Create model from the form.
@@ -941,7 +1057,7 @@ class BaseModelView(BaseView, ActionsMixin):
             :param form:
                 Form instance
         """
-        raise NotImplemented()
+        raise NotImplementedError()
 
     def update_model(self, form, model):
         """
@@ -956,7 +1072,7 @@ class BaseModelView(BaseView, ActionsMixin):
             :param model:
                 Model instance
         """
-        raise NotImplemented()
+        raise NotImplementedError()
 
     def delete_model(self, model):
         """
@@ -969,7 +1085,7 @@ class BaseModelView(BaseView, ActionsMixin):
             :param model:
                 Model instance
         """
-        raise NotImplemented()
+        raise NotImplementedError()
 
     # Various helpers
     def _prettify_name(self, name):
@@ -1006,7 +1122,9 @@ class BaseModelView(BaseView, ActionsMixin):
                     value = request.args[n]
 
                     if flt.validate(value):
-                        filters.append((pos, (idx, flt.clean(value))))
+                        filters.append((pos, (idx, flt.name, value)))
+                    else:
+                        flash(gettext('Invalid Filter Value: %(value)s', value=value))
 
             # Sort filters
             return [v[1] for v in sorted(filters, key=lambda n: n[0])]
@@ -1017,50 +1135,39 @@ class BaseModelView(BaseView, ActionsMixin):
         """
             Return arguments from query string.
         """
-        page = request.args.get('page', 0, type=int)
-        sort = request.args.get('sort', None, type=int)
-        sort_desc = request.args.get('desc', None, type=int)
-        search = request.args.get('search', None)
-        filters = self._get_list_filter_args()
+        return ViewArgs(page=request.args.get('page', 0, type=int),
+                        sort=request.args.get('sort', None, type=int),
+                        sort_desc=request.args.get('desc', None, type=int),
+                        search=request.args.get('search', None),
+                        filters=self._get_list_filter_args())
 
-        return page, sort, sort_desc, search, filters
-
-    def _get_url(self, view=None, page=None, sort=None, sort_desc=None,
-                 search=None, filters=None):
+    # URL generation helpers
+    def _get_list_url(self, view_args):
         """
             Generate page URL with current page, sort column and
             other parameters.
 
             :param view:
                 View name
-            :param page:
-                Page number
-            :param sort:
-                Sort column index
-            :param sort_desc:
-                Use descending sorting order
-            :param search:
-                Search query
-            :param filters:
-                List of active filters
+            :param view_args:
+                ViewArgs object with page number, filters, etc.
         """
-        if not search:
-            search = None
+        page = view_args.page or None
+        desc = 1 if view_args.sort_desc else None
 
-        if not page:
-            page = None
+        kwargs = dict(page=page, sort=view_args.sort, desc=desc, search=view_args.search)
+        kwargs.update(view_args.extra_args)
 
-        kwargs = dict(page=page, sort=sort, desc=sort_desc, search=search)
-
-        if filters:
-            for i, pair in enumerate(filters):
-                idx, value = pair
+        if view_args.filters:
+            for i, pair in enumerate(view_args.filters):
+                idx, flt_name, value = pair
 
                 key = 'flt%d_%s' % (i, self.get_filter_arg(idx, self._filters[idx]))
                 kwargs[key] = value
 
-        return url_for(view, **kwargs)
+        return self.get_url('.index_view', **kwargs)
 
+    # Actions
     def is_action_allowed(self, name):
         """
             Override this method to allow or disallow actions based
@@ -1099,7 +1206,11 @@ class BaseModelView(BaseView, ActionsMixin):
         if choices_map:
             return choices_map.get(value) or value
 
-        type_fmt = self.column_type_formatters.get(type(value))
+        type_fmt = None
+        for typeobj, formatter in self.column_type_formatters.items():
+            if isinstance(value, typeobj):
+                type_fmt = formatter
+                break
         if type_fmt is not None:
             value = type_fmt(self, value)
 
@@ -1128,7 +1239,7 @@ class BaseModelView(BaseView, ActionsMixin):
         """
             Model backend will override this to implement AJAX model loading.
         """
-        raise NotImplemented()
+        raise NotImplementedError()
 
     # Views
     @expose('/')
@@ -1137,16 +1248,16 @@ class BaseModelView(BaseView, ActionsMixin):
             List view
         """
         # Grab parameters from URL
-        page, sort_idx, sort_desc, search, filters = self._get_list_extra_args()
+        view_args = self._get_list_extra_args()
 
         # Map column index to column name
-        sort_column = self._get_column_by_idx(sort_idx)
+        sort_column = self._get_column_by_idx(view_args.sort)
         if sort_column is not None:
             sort_column = sort_column[0]
 
         # Get count and data
-        count, data = self.get_list(page, sort_column, sort_desc,
-                                    search, filters)
+        count, data = self.get_list(view_args.page, sort_column, view_args.sort_desc,
+                                    view_args.search, view_args.filters)
 
         # Calculate number of pages
         num_pages = count // self.page_size
@@ -1159,21 +1270,25 @@ class BaseModelView(BaseView, ActionsMixin):
             if p == 0:
                 p = None
 
-            return self._get_url('.index_view', p, sort_idx, sort_desc,
-                                 search, filters)
+            return self._get_list_url(view_args.clone(page=p))
 
         def sort_url(column, invert=False):
             desc = None
 
-            if invert and not sort_desc:
+            if invert and not view_args.sort_desc:
                 desc = 1
 
-            return self._get_url('.index_view', page, column, desc,
-                                 search, filters)
+            return self._get_list_url(view_args.clone(sort=column, sort_desc=desc))
 
         # Actions
         actions, actions_confirmation = self.get_actions_list()
 
+        clear_search_url = self._get_list_url(view_args.clone(page=0,
+                                                              sort=view_args.sort,
+                                                              sort_desc=view_args.sort_desc,
+                                                              search=None,
+                                                              filters=None))
+
         return self.render(self.list_template,
                                data=data,
                                # List
@@ -1183,32 +1298,24 @@ class BaseModelView(BaseView, ActionsMixin):
                                enumerate=enumerate,
                                get_pk_value=self.get_pk_value,
                                get_value=self.get_list_value,
-                               return_url=self._get_url('.index_view',
-                                                        page,
-                                                        sort_idx,
-                                                        sort_desc,
-                                                        search,
-                                                        filters),
+                               return_url=self._get_list_url(view_args),
                                # Pagination
                                count=count,
                                pager_url=pager_url,
                                num_pages=num_pages,
-                               page=page,
+                               page=view_args.page,
                                # Sorting
-                               sort_column=sort_idx,
-                               sort_desc=sort_desc,
+                               sort_column=view_args.sort,
+                               sort_desc=view_args.sort_desc,
                                sort_url=sort_url,
                                # Search
                                search_supported=self._search_supported,
-                               clear_search_url=self._get_url('.index_view',
-                                                              None,
-                                                              sort_idx,
-                                                              sort_desc),
-                               search=search,
+                               clear_search_url=clear_search_url,
+                               search=view_args.search,
                                # Filters
                                filters=self._filters,
                                filter_groups=self._filter_groups,
-                               active_filters=filters,
+                               active_filters=view_args.filters,
 
                                # Actions
                                actions=actions,
@@ -1219,18 +1326,18 @@ class BaseModelView(BaseView, ActionsMixin):
         """
             Create model view
         """
-        return_url = get_redirect_target() or url_for('.index_view')
+        return_url = get_redirect_target() or self.get_url('.index_view')
 
         if not self.can_create:
             return redirect(return_url)
 
         form = self.create_form()
 
-        if validate_form_on_submit(form):
+        if self.validate_form(form):
             if self.create_model(form):
                 if '_add_another' in request.form:
-                    flash(gettext('Model was successfully created.'))
-                    return redirect(url_for('.create_view', url=return_url))
+                    flash(gettext('Record was successfully created.'))
+                    return redirect(request.url)
                 else:
                     return redirect(return_url)
 
@@ -1247,7 +1354,7 @@ class BaseModelView(BaseView, ActionsMixin):
         """
             Edit model view
         """
-        return_url = get_redirect_target() or url_for('.index_view')
+        return_url = get_redirect_target() or self.get_url('.index_view')
 
         if not self.can_edit:
             return redirect(return_url)
@@ -1263,14 +1370,17 @@ class BaseModelView(BaseView, ActionsMixin):
 
         form = self.edit_form(obj=model)
 
-        if validate_form_on_submit(form):
+        if self.validate_form(form):
             if self.update_model(form, model):
                 if '_continue_editing' in request.form:
-                    flash(gettext('Model was successfully saved.'))
+                    flash(gettext('Record was successfully saved.'))
                     return redirect(request.url)
                 else:
                     return redirect(return_url)
 
+        if request.method == 'GET':
+            self.on_form_prefill(form, id)
+
         form_opts = FormOpts(widget_args=self.form_widget_args,
                              form_rules=self._form_edit_rules)
 
@@ -1285,7 +1395,7 @@ class BaseModelView(BaseView, ActionsMixin):
         """
             Delete model view. Only POST method is allowed.
         """
-        return_url = get_redirect_target() or url_for('.index_view')
+        return_url = get_redirect_target() or self.get_url('.index_view')
 
         # TODO: Use post
         if not self.can_delete:
@@ -1322,4 +1432,4 @@ class BaseModelView(BaseView, ActionsMixin):
             abort(404)
 
         data = [loader.format(m) for m in loader.get_list(query, offset, limit)]
-        return Response(json.dumps(data), mimetype='application/json')
+        return Response(json.dumps(data), mimetype='application/json')
\ No newline at end of file
